
(defvar media-last-line nil "Last cursor position in removable media list buffer")
(defvar media-current nil "Current shown removable media device")
(defvar media-mode-map nil "Key map for removable media list mode")

(let ((map (make-sparse-keymap)))
  (suppress-keymap map t)
  (define-key map "v" 'media-view)
  (define-key map "b" 'media-back)
  (define-key map "g" 'media-refresh)
  (define-key map "m" 'media-mount)
  (define-key map "\r" 'media-handle)
  (setq media-mode-map map))

(require 'string)

(defun media-device-by-line (line)
  (let ((ln) (q))
      (setq ln (- line 1))
      (setq q media-devices)
      (while (not (equal ln 0))
        (setq q (cdr q))
        (setq ln (- ln 1)))
      (car q)))

(defun media-mount-device (device)
  (let ((s))
    (setq s (car (split-string (shell-command-to-string (format "emacs-media-mount \'%s\' 2> /dev/null" device)) "\n")))
(message s)
(when (> (length s) 0)
(find-file s))
(> (length s) 0)))

(defun media-fill-initial ()
  (setq buffer-read-only nil)
  (erase-buffer)
  (shell-command "emacs-media-list 2> /dev/null" "media")
  (setq buffer-read-only t)
  (when media-last-line
    (goto-line media-last-line))
  (setq media-devices (split-string (shell-command-to-string "emacs-media-list --names-only 2> /dev/null") "\n")))

(defun media-fill-part ()
  (setq buffer-read-only nil)
  (erase-buffer)
  (when (= (shell-command (format "emacs-media-part \'%s\' 2> /dev/null" media-current) "media") 0)
  (setq buffer-read-only t)
  (goto-line 1)
  (setq media-devices (split-string (shell-command-to-string (format "emacs-media-part --names-only \'%s\' 2> /dev/null" media-current)) "\n")) t))

(defun media ()
  "Removable media device manager"
  (interactive)
  (set-buffer (get-buffer-create "media"))
  (switch-to-buffer "media")
  (setq media-last-line nil)
  (setq media-current nil)
  (media-fill-initial)
  (setq major-mode 'media-mode)
  (setq mode-name "media-mode")
  (use-local-map media-mode-map))

(defun media-view ()
  (interactive)
  (when (not media-current)
    (setq media-last-line (+ (count-lines 1 (point)) (if (= (current-column) 0) 1 0)))
    (let ((q))
      (setq q (media-device-by-line media-last-line))
      (when q (when (> (length q) 0)
	(setq media-current q)
	(when (not (media-fill-part))
	  (setq media-current nil)
	  (media-fill-initial)))))))

(defun media-refresh ()
  (interactive)
  (if (not media-current)
      (media-fill-initial)
    (when (not (media-fill-part))
      (setq media-current nil)
      (media-fill-initial))))

(defun media-back ()
  (interactive)
  (setq media-current nil)
  (media-fill-initial)
  (setq media-last-line nil))

(defun media-mount ()
  (interactive)
  (let ((device))
    (setq device (media-device-by-line (+ (count-lines 1 (point)) (if (= (current-column) 0) 1 0))))
    (when device (when (> (length device) 0)
      (when (media-mount-device device)
	(kill-buffer "media"))))))

(defun media-handle ()
  (interactive)
  (if media-current
    (media-mount)
    (let ((s))
      (setq s (media-device-by-line (+ (count-lines 1 (point)) (if (= (current-column) 0) 1 0))))
      (when s (when (> (length s) 0)
	(setq s (car (split-string (shell-command-to-string (format "emacs-media-select \'%s\' 2> /dev/null" s)) "\n")))
	(if (> (length s) 0)
	  (when (media-mount-device s) (kill-buffer "media"))
	  (media-view)))))))

(provide 'removable-media)
